@@ -1,19 +1,21 @@
 """
 Package name: 'rnt' (Reddit Network Toolkit)
-Version number: 0.1.0,
-Author: Jacob A. Rohde,
+Version number: 0.1.3 (released 10/21/2022)
+Author: Jacob A. Rohde
 Author_email: jarohde1@gmail.com
-Description: A simple tool for generating and analyzing Reddit networks.
+Description: A simple tool for generating and analyzing Reddit networks
 Github url: https://github.com/jarohde/rnt
 License: MIT
 """
 
-__all__ = ['GetRedditData', 'GetRedditNetwork', 'subreddit_statistics',
-           'reddit_thread_statistics', 'merge_reddit_submissions_and_comments']
+__all__ = ['GetRedditData', 'GetRedditNetwork', 'subreddit_statistics', 'reddit_thread_statistics',
+           'merge_reddit_submissions_and_comments', 'single_network_plot']
+
 import pandas as pd
+import networkx as nx
+import numpy as np
 from pmaw import PushshiftAPI
 from datetime import datetime, timedelta
-import networkx as nx
 from math import nan
 pd.options.mode.chained_assignment = None
 
@@ -22,7 +24,7 @@ class GetRedditData:
     """
     A class object for extracting a Reddit data set.
 
-    Parameters:
+    Arguments:
 
     - search_term: The only required parameter; takes a string object (or list of strings) as a keyword for
       searching Reddit submissions.
@@ -45,7 +47,7 @@ class GetRedditData:
     Methods:
 
     - GetRedditData.write_data(): Object method that writes the pandas DataFrame object to file. The method can take
-      file_type and file_name as optional parameters. file_type indicates what file format to use when writing the data
+      file_type and file_name as optional arguments. file_type indicates what file format to use when writing the data
       set and accepts a string argument of either 'json' or 'csv'; default set to 'json'. file_name takes a string to
       indicate what the file name should be saved as; default set to the search term provided.
     """
@@ -94,7 +96,7 @@ class GetRedditData:
             collection_type = 'search term(s).'
 
         if limit is None:
-            search_string_prompt = f'Collecting all submissions and comments from the "{self.search_term}" ' \
+            search_string_prompt = f'Collecting all submissions and their comments from the "{self.search_term}" ' \
                                    f'{collection_type}'
 
         else:
@@ -113,23 +115,31 @@ class GetRedditData:
         else:
             submission_kwargs['q'] = self.search_term
 
-        submissions_data = api.search_submissions(**submission_kwargs)
+        submissions_api_response = api.search_submissions(**submission_kwargs)
+        submissions_data = [row for row in submissions_api_response]
+        submission_ids = [row['id'] for row in submissions_data]
 
-        submissions_data = [row for row in submissions_data]
+        comment_calls = 500
+        comment_bins = [submission_ids[i:i + comment_calls] for i in range(0, len(submission_ids), comment_calls)]
 
-        submission_ids = [row['id'] for row in submissions_data if row['num_comments'] > 0]
+        comments_data = []
+        bin_index = 1
 
-        comment_kwargs = {'q': '*', 'link_id': submission_ids, 'mem_safe': True}
+        for comment_bin in comment_bins:
+            print(f'Collecting comments (batch {bin_index} of {len(comment_bins)}).')
+            comment_kwargs = {'q': '*', 'link_id': comment_bin, 'mem_safe': True}
+            comments_api_response = api.search_comments(**comment_kwargs)
 
-        comments_data = api.search_comments(**comment_kwargs)
+            for comment in comments_api_response:
+                comments_data.append(comment)
 
-        comments_data = [comment for comment in comments_data]
+            bin_index += 1
 
         collected_data_list = submissions_data + comments_data
 
         title, body, selftext, author, score, created_utc, id, \
-            link_id, parent_id, subreddit, num_comments, distinguished, post_type = \
-            [], [], [], [], [], [], [], [], [], [], [], [], []
+            link_id, parent_id, subreddit, post_type = \
+            [], [], [], [], [], [], [], [], [], [], []
 
         for row in collected_data_list:
             author.append(row['author'])
@@ -142,19 +152,15 @@ class GetRedditData:
                 title.append('')
                 selftext.append('')
                 body.append(row['body'])
-                num_comments.append('')
                 link_id.append(row['link_id'])
                 parent_id.append(row['parent_id'])
-                distinguished.append(row['distinguished'])
                 post_type.append('c')
 
             else:
                 title.append(row['title'])
                 body.append('')
-                num_comments.append(row['num_comments'])
                 link_id.append('')
                 parent_id.append('')
-                distinguished.append('')
                 post_type.append('s')
                 try:
                     selftext.append(row['selftext'])
@@ -162,14 +168,14 @@ class GetRedditData:
                     selftext.append('')
 
         df = pd.DataFrame(list(zip(title, body, selftext, author, score, created_utc, id,
-                                   link_id, parent_id, subreddit, num_comments, distinguished, post_type)),
+                                   link_id, parent_id, subreddit, post_type)),
                           columns=['title', 'body', 'selftext', 'author', 'score', 'created_utc', 'id',
-                                   'link_id', 'parent_id', 'subreddit', 'num_comments', 'distinguished', 'post_type'])
+                                   'link_id', 'parent_id', 'subreddit', 'post_type'])
 
         df['created'] = df.created_utc.apply(lambda x: datetime.utcfromtimestamp(x))
 
-        df = df[['author', 'title', 'selftext', 'body', 'id', 'subreddit', 'num_comments', 'parent_id',
-                 'score', 'distinguished', 'link_id', 'created_utc', 'created', 'post_type']]
+        df = df[['author', 'title', 'selftext', 'body', 'id', 'subreddit', 'parent_id',
+                 'score', 'link_id', 'created_utc', 'created', 'post_type']]
 
         df = df.sort_values(by='created_utc')
         df = df.reset_index(drop=True)
@@ -191,7 +197,7 @@ class GetRedditData:
         """
         A method for saving Reddit data to file.
 
-        Parameters:
+        Arguments:
 
         - file_type: Accepts a string of either 'json' or 'csv'; default set to 'json'.
 
@@ -220,7 +226,7 @@ class GetRedditNetwork:
     """
     A class object for generating edge and node lists, and a NetworkX graph object from a Reddit data set.
 
-    Parameters:
+    Arguments:
 
     - reddit_dataset: The only required argument. Takes a Reddit data set or a GetRedditData object.
 
@@ -245,13 +251,19 @@ class GetRedditNetwork:
     - GetRedditNetwork.node_list: Returns a pandas DataFrame of the network node list with columns for each unique node,
       the node's in-degree and out-degree values, and a list of subreddits the node participated in within the network.
 
+    - GetRedditData.adjacency: Returns a dictionary of network adjacency matrices. By default, both weighted and
+      unweighted matrices are returned. The dictionary will also return weighted adjacency matrices for each optional
+      edge-based text attribute that users identified when creating the class.
+
     - GetRedditNetwork.graph: Returns a NetworkX graph object.
 
     Methods:
 
-    - GetRedditData.write_data(): Object method that writes the edge_list and node_list attributes to file. The method
-      can take file_type as an optional parameter. file_type indicates what file format to use when writing the data
-      sets and accepts a string argument of either 'json' or 'csv'; default set to 'json'.
+    - GetRedditData.write_data(): Object method that writes edge_list and node_list data sets to file. The method
+      takes file_type, file_name, and adjacency as optional parameters. file_type indicates what file format to use when
+      writing the data sets and accepts a string argument of either 'json' or 'csv'; default set to 'json'. file_name
+      accepts a string to indicate what name to save the files as. adjacency accepts a boolean and indicates whether to
+      write the data sets as adjacency matrices instead of edge and node lists.
     """
 
     def __init__(self, reddit_dataset, **kwargs):
@@ -262,6 +274,7 @@ class GetRedditNetwork:
         self.edge_by = kwargs.get('edge_by', 'link_id')
         self.edge_list = pd.DataFrame
         self.node_list = pd.DataFrame
+        self.adjacency = dict
         self.graph = nx.Graph
 
         if 'GetRedditData' in str(type(self.reddit_dataset)):
@@ -334,8 +347,8 @@ class GetRedditNetwork:
             author_list = [author] * len(commenter_list)
             subreddit_list = [thread_subreddit] * len(commenter_list)
 
-            thread_edge_df = pd.DataFrame(list(zip(author_list, commenter_list, subreddit_list)),
-                             columns=['poster', 'commenter', 'subreddit'])
+            thread_edge_df = pd.DataFrame(list(zip(commenter_list, author_list, subreddit_list)),
+                             columns=['source', 'target', 'subreddit'])
 
             if self.text_attribute is not False:
                 for col in text_attribute_columns:
@@ -344,48 +357,95 @@ class GetRedditNetwork:
             edge_dfs.append(thread_edge_df)
 
         edge_list = pd.concat(edge_dfs, ignore_index=True)
-        edge_list = edge_list.loc[edge_list.commenter != '']
+        edge_list = edge_list.loc[edge_list.source != '']
 
         edge_list = edge_list.reset_index(drop=True)
 
-        in_degree_values, out_degree_values, node_subreddits = [], [], []
+        node_subreddits = []
 
         for node in nodes:
-            in_degree_values.append(len(edge_list.loc[edge_list.poster == node]))
-            out_degree_values.append(len(edge_list.loc[edge_list.commenter == node]))
             node_subreddits.append(list(df.loc[df.author == node].subreddit.unique()))
 
-        node_list = pd.DataFrame(list(zip(nodes, in_degree_values, out_degree_values, node_subreddits)),
-                                 columns=['nodes', 'in_degree', 'out_degree', 'node_subreddits'])
-
         graph_object = nx.DiGraph()
 
         for node, attribute in zip(nodes, node_subreddits):
             graph_object.add_node(node, subreddit_list=attribute)
 
-        for n1, n2 in zip(edge_list.poster, edge_list.commenter):
+        for n1, n2 in zip(edge_list.source, edge_list.target):
             graph_object.add_edge(n1, n2)
 
         attrs = {}
         edge_list_index = 0
-        for n1, n2, subreddit in zip(edge_list.poster, edge_list.commenter, edge_list.subreddit):
-            edge_attr_dict = {}
-            edge_attr_dict = {**edge_attr_dict, **{'subreddit': subreddit}}
+        for n1, n2, subreddit in zip(edge_list.source, edge_list.target, edge_list.subreddit):
+            if (n1, n2) not in attrs.keys():
+                edge_attr_dict = {}
+                edge_attr_dict = {**edge_attr_dict, **{'subreddit': subreddit,
+                                                       'weight': len(edge_list[(edge_list.source == n1) &
+                                                                               (edge_list.target == n2)])}}
 
-            if self.text_attribute is not False:
-                for col in text_attribute_columns:
-                    edge_attr_dict = {**edge_attr_dict, **{col: edge_list[col][edge_list_index]}}
+                if self.text_attribute is not False:
+                    for col in text_attribute_columns:
+                        edge_attr_dict = {**edge_attr_dict, **{col: edge_list[col][edge_list_index]}}
 
-            edge_list_index += 1
-            attrs[(n1, n2)] = edge_attr_dict
+                edge_list_index += 1
+                attrs[(n1, n2)] = edge_attr_dict
+
+            else:
+                continue
 
         nx.set_edge_attributes(graph_object, attrs)
 
+        degree, in_degree_values, out_degree_values = [], [], []
+
+        for node in nodes:
+            degree.append(graph_object.degree(node))
+            in_degree_values.append(graph_object.in_degree(node))
+            out_degree_values.append(graph_object.out_degree(node))
+
+        node_list = pd.DataFrame(list(zip(nodes, degree, in_degree_values, out_degree_values, node_subreddits)),
+                                 columns=['node', 'degree', 'in_degree', 'out_degree', 'node_subreddits'])
+
         if self.edge_type.lower() == 'undirected':
             graph_object = graph_object.to_undirected()
 
-        self.edge_list = edge_list
+        pandas_edge_list = nx.to_pandas_edgelist(graph_object)
+
+        if len(pandas_edge_list) != 0:
+            el_columns = ['source', 'target', 'weight', 'subreddit']
+            el_columns = el_columns + [column for column in edge_list.columns if column not in el_columns]
+            pandas_edge_list = pandas_edge_list[el_columns]
+
+        weighted_adj_matrix = nx.to_pandas_adjacency(graph_object)
+        np.fill_diagonal(weighted_adj_matrix.values, nan)
+
+        unweighted_adj_matrix = nx.to_pandas_adjacency(graph_object)
+        unweighted_adj_matrix[unweighted_adj_matrix > 1] = 1
+        np.fill_diagonal(unweighted_adj_matrix.values, nan)
+
+        matrices = {'weighted_adj_matrix': weighted_adj_matrix,
+                    'unweighted_adj_matrix': unweighted_adj_matrix}
+
+        text_attribute_columns = [column for column in pandas_edge_list.columns if 'text_attribute' in column]
+
+        for column in text_attribute_columns:
+            adj_matrix = nx.to_pandas_adjacency(graph_object)
+            adj_matrix[:] = 0
+
+            for u, v, attribute in zip(pandas_edge_list.source, pandas_edge_list.target, pandas_edge_list[column]):
+                if attribute == 'True':
+                    cell_loading = 1
+
+                else:
+                    cell_loading = 0
+
+                adj_matrix.at[u, v] = cell_loading
+
+            np.fill_diagonal(adj_matrix.values, nan)
+            matrices[column + '_matrix'] = adj_matrix
+
         self.node_list = node_list
+        self.edge_list = pandas_edge_list
+        self.adjacency = matrices
         self.graph = graph_object
 
     def __repr__(self):
@@ -402,46 +462,64 @@ class GetRedditNetwork:
 
         Parameter:
 
-        - file_type: Accepts a string of either 'json' or 'csv'; default set to 'json'.
+        - file_type: Accepts a string of either 'json' or 'csv'; default set to 'csv'.
 
         - file_name: Accepts a string to indicate what the edge and node list files should be saved as; default set to
           'edge_list' and 'node_list'.
+
+        - adjacency: Accepts a boolean; default set to False. If True, adjacency matrices of the network (including
+          matrices of all edge attributes) are saved as csv files.
+
         """
 
-        file_type = kwargs.get('file_type', 'json')
+        file_type = kwargs.get('file_type', 'csv')
         file_name = kwargs.get('file_name', '')
-        file_list = ['node_list', 'edge_list']
+        adjacency = kwargs.get('adjacency', False)
 
         if file_type.lower() != 'json' and file_type.lower() != 'csv' or type(file_type) != str:
             return 'Error: File type only supports .csv or .json extensions.'
 
-        for file in file_list:
-            if type(file_name) is str and file_name != '':
-                f_name = f'{file}_{file_name}'
-            else:
-                f_name = file
+        if adjacency:
+            matrices = self.adjacency
 
-            name = f'{f_name}.{file_type.lower()}'
-            print(f'Writing {f_name} to file.')
-
-            if file_type.lower() == 'json':
-                if file == 'node_list':
-                    self.node_list.to_json(name, orient='records', lines=True)
+            for matrix in matrices:
+                if type(file_name) is str and file_name != '':
+                    f_name = f'{matrix}_{file_name}.csv'
                 else:
-                    self.edge_list.to_json(name, orient='records', lines=True)
+                    f_name = f'{matrix}.csv'
 
-            if file_type.lower() == 'csv':
-                if file == 'node_list':
-                    self.node_list.to_csv(name, index=False, header=True, encoding='utf-8', na_rep='nan')
+                print(f'Writing {f_name} to file.')
+                matrices[matrix].to_csv(f_name, index=True, header=True, encoding='utf-8', na_rep='nan')
+
+        else:
+            file_list = ['node_list', 'edge_list']
+            for file in file_list:
+                if type(file_name) is str and file_name != '':
+                    f_name = f'{file}_{file_name}'
                 else:
-                    self.edge_list.to_csv(name, index=False, header=True, encoding='utf-8', na_rep='nan')
+                    f_name = file
+
+                name = f'{f_name}.{file_type.lower()}'
+                print(f'Writing {name} to file.')
+
+                if file_type.lower() == 'json':
+                    if file == 'node_list':
+                        self.node_list.to_json(name, orient='records', lines=True)
+                    else:
+                        self.edge_list.to_json(name, orient='records', lines=True)
+
+                if file_type.lower() == 'csv':
+                    if file == 'node_list':
+                        self.node_list.to_csv(name, index=False, header=True, encoding='utf-8', na_rep='nan')
+                    else:
+                        self.edge_list.to_csv(name, index=False, header=True, encoding='utf-8', na_rep='nan')
 
 
 def subreddit_statistics(reddit_dataset, subreddit_list=None):
     """
     A function for extracting basic statistics for single or batch subreddit networks.
 
-    Parameters:
+    Arguments:
 
     - reddit_dataset: The only required argument. Takes a Reddit data set or a GetRedditData object.
 
@@ -474,6 +552,7 @@ def subreddit_statistics(reddit_dataset, subreddit_list=None):
     first_subreddit = True
     for subreddit in subreddit_list:
         subreddit_df = df.loc[df.subreddit == subreddit]
+
         subreddit_graph = GetRedditNetwork(subreddit_df).graph
 
         connected_components = sorted(nx.strongly_connected_components(subreddit_graph), key=len, reverse=True)
@@ -508,7 +587,7 @@ def reddit_thread_statistics(reddit_dataset, reddit_thread_list=None):
     A function for extracting basic statistics for single or batch Reddit threads (initiated by Reddit
     submissions).
 
-    Parameters:
+    Arguments:
 
     - reddit_dataset: The only required argument. Takes a Reddit data set or a GetRedditData object.
 
@@ -581,7 +660,6 @@ def reddit_thread_statistics(reddit_dataset, reddit_thread_list=None):
                        'thread_id': thread,
                        'date': list(author_df.created)[0],
                        'subreddit': list(author_df.subreddit)[0],
-                       'distinguished': list(author_df.distinguished)[0],
                        'score': list(author_df.score)[0],
                        'num_unique_responders': num_unique_responders,
                        'earliest_response_time': earliest_response_time,
@@ -646,8 +724,8 @@ def merge_reddit_submissions_and_comments(submissions_dataset, comments_dataset)
 
     df['created'] = df.created_utc.apply(lambda x: datetime.utcfromtimestamp(int(x)))
 
-    df = df[['author', 'title', 'selftext', 'body', 'id', 'subreddit', 'num_comments', 'parent_id',
-             'score', 'distinguished', 'link_id', 'created_utc', 'created', 'post_type']]
+    df = df[['author', 'title', 'selftext', 'body', 'id', 'subreddit', 'parent_id',
+             'score', 'link_id', 'created_utc', 'created', 'post_type']]
 
     df = df.sort_values(by='created_utc')
     df = df.reset_index(drop=True)
@@ -670,3 +748,96 @@ def add_post_type_column(df):
         df['post_type'] = post_type
 
     return df
+
+
+def single_network_plot(network, **kwargs):
+
+    """
+    A simple function for plotting networks via NetworkX and Matplotlib (additional install required). Please note this
+    function is currently a work in progress and is meant to be basic tool to plot a single graph. See NetworkX
+    documentation for more advanced plotting needs.
+
+    Arguments:
+
+    - network: The only required argument. Takes a GetRedditNetwork or NetworkX graph object.
+
+    - title: Optional string argument to add a title to the plot.
+
+    - pos: Optional string argument to set the NetworkX plotting algorithm. For ease of use, the argument currently
+      accepts one of the following layout types as a string:
+
+      - 'spring_layout' (default)
+      - 'kamada_kawai_layout'
+      - 'circular_layout'
+      - 'random_layout'
+
+    - **kwargs: The function also accepts several other NetworkX keyword arguments for plotting (please see NetworkX
+      documentation for more info on these arguments). Currently accepted arguments include:
+
+      - 'arrows' (bool)
+      - 'arrowsize' (int)
+      - 'edge_color' (str or list/array)
+      - 'font_size' (int)
+      - 'node_color' (str or list/array)
+      - 'node_size' (str or list/array)
+      - 'verticalalignment' (str)
+      - 'width' (int/float or list/array)
+      - 'with_labels' (bool)
+    """
+    import matplotlib.pyplot as plt
+
+    if 'GetRedditNetwork' in str(type(network)):
+        G = network.graph
+
+    elif 'networkx.classes' in str(type(network)):
+        G = network
+
+    else:
+        G = None
+
+    # node attributes
+    with_labels = kwargs.get('with_labels', False)
+    node_color = kwargs.get('node_color', 'black')
+    node_size = kwargs.get('node_size', 30)
+    verticalalignment = kwargs.get('verticalalignment', 'center')
+    font_size = kwargs.get('font_size', 8)
+
+    # edge attributes
+    edge_color = kwargs.get('edge_color', 'grey')
+    width = kwargs.get('width', 1)
+    arrows = kwargs.get('arrows', None)
+    arrowsize = kwargs.get('arrowsize', 10)
+
+    # general plot attributes
+    title = kwargs.get('title', None)
+    pos = kwargs.get('pos', 'spring_layout')
+
+    plt.figure()
+    plt.title(title)
+
+    if G is not None:
+        layouts = {'spring_layout': nx.spring_layout(G),
+                   'kamada_kawai_layout': nx.kamada_kawai_layout(G),
+                   'circular_layout': nx.circular_layout(G),
+                   'random_layout': nx.random_layout(G)}
+
+        pos = layouts.get(pos.lower(), 'spring_layout')
+
+        plt_kwargs = {'pos': pos,
+                      'edge_color': edge_color,
+                      'node_color': node_color,
+                      'with_labels': with_labels,
+                      'verticalalignment': verticalalignment,
+                      'font_size': font_size,
+                      'node_size': node_size,
+                      'width': width,
+                      'arrows': arrows,
+                      'arrowsize': arrowsize}
+
+        nx.draw_networkx(G, **plt_kwargs)
+
+        plt.show()
+
+    else:
+        print(f'Error: network argument expected GetRedditNetwork or networkx graph object; '
+              f'received {type(network)} instead.')
